/*
 * async_server.cpp
 *
 *  Created on: 02.01.2021
 *      Author: doralitze
 */
#include "net/async_server.hpp"

#include <ev++.h>

#include <utility>


namespace rmrf::net {

async_server::async_server(
    auto_fd &&socket_fd,
    const std::shared_ptr<::ev::loop_ref>& loop_ref
) :
    socket(std::forward<auto_fd>(socket_fd)),
    loop{loop_ref ? loop_ref : std::make_shared<::ev::loop_ref>(::ev::default_loop{})},
    io{},
    number_of_connected_clients{0},
    max_number_of_simulataneusly_allowed_clients{0},
    on_accept{},
    on_overflow{},
    on_error{}
{
    // This constructor got a constructed socket as an argument
    // and forwards it to libev
    io.set(*loop);
    io.set<async_server, &async_server::cb_ev>(this);
    io.start(this->socket.get(), ::ev::READ);
}

async_server::~async_server() {
    // Remove this socket from libev ...
    io.stop();
}

void async_server::cb_ev(::ev::io &w, int events) {
    (void) w;

    if (events & ::ev::ERROR) {
        this->on_error(this->shared_from_this());
        return;
    }

    if (events & ::ev::READ) {
        // Handle incoming clients
        if(auto client = this->await_raw_socket_incomming(this->socket); client) {
            if (this->max_number_of_simulataneusly_allowed_clients == 0 || this->get_number_of_connected_clients() <= this->max_number_of_simulataneusly_allowed_clients) {
                this->number_of_connected_clients++;
                this->get_accept_handler()(this->shared_from_this(), client);
            } else if (this->on_overflow != nullptr) {
                this->on_overflow(this->shared_from_this(), client);
            }
        }
    }

    if (events & ::ev::WRITE) {
        // Handle sending data which should be none here
    }
}

}
